/*============================================================================

                        PVK40 Example : Serial EEPROM 93xx (SW)

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Insert 93LC46B into 93xx socket
 2) Switch off JP1 jumper
 3) Switch off all DIP switches except:
 4) Switch on OSC 3.276M on S9
 5) Switch on B0,1,2,4 DISP on S11
 6) Switch on B3 LED on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

typedef unsigned int clock_t;

static bit led @ PORTBIT(PORTB,3);
static bit eecs @ PORTBIT(PORTA,5);
static bit eedi @ PORTBIT(PORTC,4);
static bit eedo @ PORTBIT(PORTC,5);
static bit eeclk @ PORTBIT(PORTC,3);
static bit pushbutton0 @ PORTBIT(PORTD,0);

char digit[5] = {0xFF,0xFE,0xFD,0xFB,0xEF}; 
char segment[10] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90};

clock_t systimer;         /* systimer is incremented every 20ms */
clock_t now;
clock_t lap;
unsigned int period;
unsigned int count;

/*============================================================================

       Program

============================================================================*/

void interrupt timer0(void)
{
  T0IF=0;
  systimer++;
}

clock_t clock(void)        /* read systimer */
{
  clock_t temp;

  GIE=0;
  temp=systimer;
  GIE=1;
  return temp;
}

/* dispdigit(j,p) - shows the value of an integer j
   on one p of the 7 segments display            */ 
void dispdigit(int j, int p)
{
  PORTB=(digit[0]&0xF7)|(PORTB&0x08);
  PORTD=segment[j];
  TRISD=0x00;
  PORTB=(digit[p]&0xF7)|(PORTB&0x08);
}

void timer0init(void)
{    /* initialisation of timer0 */
  PS2=1;  PS1=0;  PS0 = 1; /* select prescaler by 64 */
  T0CS = 0;                /* select timer0 */
  PSA=0;                   /* select prescaler for timer0 */
  TMR0 =0;                 /* preload timer0 */
  T0IE = 1;                /* interrupts are actived */
}

void portinit(void)
{
  PORTA=0x00;
  PORTB=0xF7;
  PORTC=0x00;
  PORTD=0xFF;
  ADCON1=0x07;
  TRISA=0xDF;
  TRISB=0xE0;
  TRISC=0xD7;
}

unsigned char eep_xfer(unsigned char data)
{   /* transfer data between PIC and EEPROM */
  unsigned char i, buffer;

  buffer=0x00;
  for(i=8;i;i--) {   /* eight bits */
    if (data&0x80)
      eedo=1;
    else
      eedo=0;
    data<<=1;
    eeclk=1;    /*   set clock high */
    asm("nop");
    eeclk=0;    /* set clock low */
    buffer<<=1;
    if (eedi)   /* read input data */
      buffer|=0x01;
  }

  return buffer;
}

void eep_write_enable(void)
{
  eecs=1;  /* set chip select line high */
  eep_xfer(0x01);
  eep_xfer(0x30);
  eecs=0;  /* set chip select line low */
}

void eep_wrt(unsigned char address, unsigned int data)
{
  eecs=1;  /* set chip select line high */
  eep_xfer(0x01);
  eep_xfer((address&0x3F)|0x40);
  eep_xfer(data>>8);    /* data 15-8 */
  eep_xfer(data&0xFF);  /* data 7-0 */
  eecs=0;  /* set chip select line low */
  asm("nop"); /* EECS must be low for > 250 ns */
  for(eecs=1;!eedi;eecs=0); /* wait for done */
}

unsigned int eep_read(unsigned char address)
{
  unsigned int tempdata;

  eecs=1;  /* set chip select line high */
  eep_xfer(0x01);
  eep_xfer((address&0x3F)|0x80);
  tempdata=eep_xfer(0)<<8;
  tempdata|=eep_xfer(0);
  eecs=0;  /* set chip select line low */
  return tempdata;
}

void eep_write(unsigned char address, unsigned int data)
{
  if (eep_read(address)!=data)
    eep_wrt(address,data);
}

void delayus(unsigned char us)
{
  for (;us--;);
}

void main(void)
{

  portinit();
  timer0init();
  period=50;               /* 1 second */
  GIE=1;                   /* interrupts are actived */

  count=eep_read(0x00);
  eep_write_enable();      /* allow write */

  for(;;) {
    now=clock();
    if ((now-lap)>period) {
      lap+=period;
      if(++count>9999)     /*counting*/
        count=0;
    }
    dispdigit(count%10,1);
    delayus(250);
    dispdigit((count/10)%10,2);
    delayus(250);
    dispdigit((count/100)%10,3);
    delayus(250);
    dispdigit(count/1000,4);
    delayus(250);

    TRISD=0xFF;   /* inputs */
    PORTB=digit[0];
    delayus(5);
    if(!pushbutton0) {
      led=1;
      eep_write(0,count);
    }
    else
      led=0;
  } // end of the for loop
}
