/*============================================================================

                        PVK40 Example : Alarm clock

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on B0,1,2,4 DISP on S11
 4) Switch on B3 LED on S11
 5) Switch on C2 BUZZER on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

typedef unsigned int clock_t;

static bit led @ PORTBIT(PORTB,3);

char digit[5] = {0xFF,0xFE,0xFD,0xFB,0xEF}; 
char segment[10] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90};

clock_t systimer;         /* systimer is incremented every 2.5ms */
clock_t lap1,lap2,lap3;
unsigned int period1,period2,period3;

unsigned char time_hrs=0; /* real time */
unsigned char time_min=0;
unsigned char alarm_hrs=12; /* alarm time */
unsigned char alarm_min=0;

char dot=0;
char show_alarm_time=0;
char alarmtime=0;
volatile unsigned char buttonsbuffer=0xFF;
unsigned char disppos=0;

/*============================================================================

       Program

============================================================================*/

void timer0init(void)
{
 /* initialisation of timer0; */
	PS2=0; PS1=1; PS0=0; /* select prescaler by 8 */
	T0CS=0; /* select timer0 */
	PSA=0;  /* select prescaler for timer0 */
	TMR0=0; /* preload timer0 */
	T0IE=1; /* interrupts are actived */
}

void timer1init(void)
{
 /* initialisation of timer1 */
	T1CKPS1=1; T1CKPS0=0; /* select prescaler by 4 */
	TMR1CS=0; /* select clock source */
	TMR1ON=1;  /* select prescaler for timer0 */
}

void pwminit(void)
{
  TRISC=0xFB; /* RC2 is output */
  TMR2=0;
  TMR2ON=1;
  PR2=0xFF;
  CCPR1H=0;
  CCPR1L=0;
  CCP1CON=0x0F; /* PWM mode */
}

unsigned char getportd(void)
{ /* return buttonsbuffer */
  unsigned char tmp;

  GIE=0;
  tmp=buttonsbuffer;
  GIE=1;
  return tmp;
}

void buttons_wait(void) /* wait for release all buttons */
{
  while(getportd()!=0xFF) ;
}

void settime(char hrs,char min)
{
  unsigned char *p_hrs, *p_min;

  if(show_alarm_time) { /* select real time or alarm time */
    p_hrs=&alarm_hrs;
    p_min=&alarm_min;
  }
  else {
    p_hrs=&time_hrs;
    p_min=&time_min;
  }

  GIE=0;
  *p_hrs+=hrs;
  *p_min+=min;
  if(*p_hrs>0x80)
    *p_hrs+=24;
  else if(*p_hrs>23)
    *p_hrs-=24;
  if(*p_min>0x80)
    *p_min+=60;
  else if(*p_min>59)
    *p_min-=60;
  GIE=1;
}

void alarm(void)
{
  unsigned char count;

  for(count=35;count;count--) {
    CCPR1L=0x80; /* PWM on */
    TMR1H=0;
    TMR1IF=0;
    while(!TMR1IF) { /* wait for timer 1 overflow */
      if(getportd()!=0xFF) { /* test buttons */
        buttons_wait(); /* wait for release */
        CCPR1L=0x00; /* PWM off */
        return;
      }
    }
    CCPR1L=0x00; /* PWM off */
    TMR1H=0;
    TMR1IF=0;
    while(!TMR1IF) { /* wait for timer 1 overflow */
      if(getportd()!=0xFF) { /* test buttons */
        buttons_wait(); /* wait for release */
        return;
      }
    }
  }
}

void disprefresh(unsigned char hrs,unsigned char min,char dispdot)
{
  unsigned char disptmp;

  switch (disppos) {
    case 0 :
      disptmp=segment[min%10];
      break;
    case 1 :
      disptmp=segment[(min/10)%10];
      break;
    case 2 :
      disptmp=segment[hrs%10];
      if(dispdot)
        disptmp&=0x7F;
      break;
    case 3 :
      disptmp=segment[(hrs/10)%10];
      break;
  }
  PORTB=(digit[0]&0xF7)|(PORTB&0x08);
  PORTD=disptmp;
  PORTB=(digit[disppos+1]&0xF7)|(PORTB&0x08);
  disppos=(++disppos)&3;
}

void interrupt timer0(void)
{
  systimer++;
  T0IF=0;
  if ((systimer-lap1)>period1) {
    lap1+=period1;
    if(++time_min>=60) { /* increment minutes */
      time_min=0;
      if(++time_hrs>=24)  /* increment hours */
        time_hrs=0;
    }
    if((time_min==alarm_min)&(time_hrs==alarm_hrs))
      alarmtime=1;
  }
  if ((systimer-lap2)>period2) {
    lap2+=period2;
    PORTB=(digit[0]&0xF7)|(PORTB&0x08);
    TRISD=0xFF;
    asm("nop");
    asm("nop");
    buttonsbuffer=PORTD;  /* read buttons state */
    TRISD=0x00;
  }
  if ((systimer-lap3)>period3) { /* blinking dot */
    lap3+=period3;
    dot=!dot;
  }
  if(show_alarm_time) /* refresh display - alarm time */
    disprefresh(alarm_hrs,alarm_min,1);
  else /* refresh display - real time */
    disprefresh(time_hrs,time_min,dot);
}
 
int main(void)
{
  TRISB=0xE0;
  timer0init();
  timer1init();
  pwminit();
  led=0;
  period1=24000;    /* 2.5ms*24000 = 1 minute */
  period2=20;   /* 2.5ms*20 = 50ms - buttonsbuffer refresh time */
  period3=200;   /* 2.5ms*200 = 500ms - blinking dot period */
  GIE=1;

  for(;;)
  {
    if(!(getportd()&1)) {
      led=!led;
      show_alarm_time=!show_alarm_time;
      buttons_wait();
    }

    if(!(getportd()&0x10)) {
      settime(1,0);
      buttons_wait();
    }

    if(!(getportd()&0x20)) {
      settime(-1,0);
      buttons_wait();
    }

    if(!(getportd()&0x40)) {
      settime(0,1);
      buttons_wait();
    }

    if(!(getportd()&0x80)) {
      settime(0,-1);
      buttons_wait();
    }

    if(alarmtime) {
      alarmtime=0;
      alarm();
    }
  }
}
