/*============================================================================

                        PVK40 Example : Security alarm

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on TEMP on S9
 4) Switch on PHOTO on S9
 5) Switch on MIC on S9
 6) Switch on B3 LED on S11
 7) Switch on C2 BUZZER on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

#define IN_MIC 4
#define IN_TEMP 0
#define IN_PHOTO 1
#define TOL_MIC 0x50
#define TOL_TEMP 0x02
#define TOL_PHOTO 0x40

static bit led @ PORTBIT(PORTB,3);

char btnmask[8] = {0xFE,0xFD,0xFB,0xF7,0xEF,0xDF,0xBF,0x7F};

unsigned char intcycle;
char startalarm=0;
char alarmed=0;
unsigned int ledblink;

unsigned int mic_peak; /* new sampled values */
unsigned int mic_down;
unsigned int photo_new;
unsigned int temp_new;
unsigned int mic_ref; /* reference values */
unsigned int photo_ref;
unsigned int temp_ref;

/* EEPROM data - alarm access code, only first 4 digits */
__EEPROM_DATA(0,1,2,3,0xFF,0xFF,0xFF,0xFF); /* button0,1,2 and last button3 */

/*============================================================================

       Program

============================================================================*/

void timer0init(void)
{
 /* initialisation of timer0; */
	PS2=0; PS1=0; PS0=1; /* select prescaler by 4 */
	T0CS=0; /* select timer0 */
	PSA=0;  /* select prescaler for timer0 */
	TMR0=0; /* preload timer0 */
	T0IE=1; /* interrupts are actived */
}

void timer1init(void)
{
 /* initialisation of timer1 */
	T1CKPS1=1; T1CKPS0=0; /* select prescaler by 4 */
	TMR1CS=0; /* select clock source */
	TMR1ON=1;  /* select prescaler for timer0 */
}

void pwminit(void)
{
  TRISC=0xFB; /* RC2 is output */
  TMR2=0;
  TMR2ON=1;
  T2CKPS0=1;
  PR2=0xFF;
  CCPR1H=0;
  CCPR1L=0;
  CCP1CON=0x0F; /* PWM mode */
}

void adcinit(void)
{
  /* initialisation of AD converter */
  TRISA = 0xFF;	/* all pin of portA are inputs */
  ADCON1 = 0x80;  /*config in 5/0 */
  ADCON0 = 0x51; /* config the converter  */
}

void beep(void)
{
  unsigned int counter;

  PR2=0x40;
  CCPR1L=0x02;
  for(counter=10000;counter;counter--) ; /* wait */
  CCPR1L=0x00;
}

void delayus(unsigned char us)
{
  for (;us--;);
}

unsigned int adcsample(unsigned char channel)
{
  ADCON0 = (ADCON0 & 0xc7) | (channel << 3);
  delayus(20);

  for (ADCON0|=0x04; ADCON0&0x04; );

  return ADRESL|(ADRESH<<8);
}

unsigned char getbutton(void) /* wait for button "click" */
{
  unsigned char porttmp;

  do {
    porttmp=PORTD;
  }
  while(porttmp==0xFF) ;
  beep();
  while(PORTD!=0xFF) continue;
  return porttmp;
}

unsigned char enter_key(void)
{
  char i;
  unsigned char valid=1;

  for(i=0;i<4;i++) {
    if(btnmask[EEPROM_READ(i)]!=getbutton())
      valid=0;
  }
  return valid;
}

void alarm(void)
{
  unsigned char count;

  for(count=15;count;count--) {
    PR2=0x40;
    CCPR1L=0x20; /* PWM on */
    TMR1H=0;
    TMR1IF=0;
    while(!TMR1IF) ; /* wait for timer 1 overflow */
    PR2=0x80;
    CCPR1L=0x40;
    TMR1IF=0;
    while(!TMR1IF) ; /* wait for timer 1 overflow */
  }
  CCPR1L=0x00; /* PWM off */
  TMR1IF=0;
  while(!TMR1IF) ; /* wait for timer 1 overflow */
}

void delay(void) /* delay with blinking led */
{
  unsigned char count;

  for(count=12;count;count--) {
    led=1;
    TMR1H=0;
    TMR1IF=0;
    while(!TMR1IF) ; /* wait for timer 1 overflow */
    led=0;
    TMR1IF=0;
    while(!TMR1IF) ; /* wait for timer 1 overflow */
  }
}

void protect_start(void)
{
  GIE=0;
  alarmed=1;
  GIE=1;
}

void protect_stop(void)
{
  GIE=0;
  alarmed=0;
  GIE=1;
}

char compare(unsigned int new,unsigned int ref,unsigned char tol)
{
  unsigned char tol2;

  tol2=tol;
  if(tol2>ref)
    tol2=ref;
  if((new<=(ref+tol))&(new>=(ref-tol2)))
    return 1;
  else
    return 0;
}

void interrupt timer0(void)
{
  unsigned int sample;

  T0IF=0;
  if(startalarm) {
    alarm();
    startalarm=0;
  }

  switch(alarmed) {
    case 0 :
      return;
    case 1 :
      intcycle=0;
      alarmed=2;
      break;
    case 4 :
      ledblink=0x12FF;
      alarmed=5;
      return;
    case 5 :
      ledblink--;
      if(!ledblink) {
        alarm();
        alarmed=2;
        intcycle=0;
        led=1;
      }
      else {
        if(ledblink&0x100)
          led=1;
        else
          led=0;
      }
      return;
  }

  switch(++intcycle) {
    case 1 :
      mic_peak=0x0000;
      mic_down=0xFFFF;
      break;
    case 2 :
      temp_new=adcsample(IN_TEMP);
      if(alarmed==3) {
        if(!compare(temp_new,temp_ref,TOL_TEMP))
          alarmed=4;
      }
      else {
        temp_ref=temp_new;
      }
      break;
    case 3 :
      photo_new=adcsample(IN_PHOTO);
      if(alarmed==3) {
        if(!compare(photo_new,photo_ref,TOL_PHOTO))
          alarmed=4;
      }
      else {
        photo_ref=photo_new;
      }
      break;
    case 0xFF :
      if(alarmed==3) {
        if(!compare(mic_peak-mic_down,mic_ref,TOL_MIC))
          alarmed=4;
      }
      else {
        mic_ref=mic_peak-mic_down;
        alarmed=3;
      }
      break;
    default :
      sample=adcsample(IN_MIC);
      if(sample>mic_peak)  /* compare with limits */
        mic_peak=sample;
      if(sample<mic_down)
        mic_down=sample;
      break;
  }
}
 
int main(void)
{
  TRISB=0xF7;
  led=0;
  timer0init();
  timer1init();
  pwminit();
  adcinit();
  GIE=1;

  for(;;)
  {
    if(enter_key()) {
      delay();
      protect_start();
      led=1;
      while(!enter_key()) {
        startalarm=1;
      }
      protect_stop();
      led=0;
    }
    else {
      startalarm=1;
    }
  }
}
