/*============================================================================

                        PVK40 Example : Infra-Red Receiver

 Compiler/IDE : PICC-Lite and MPLAB
 Note: UIR remote decoder, compatible with IRAMP/IRMAN

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none

 On your PVK40 board :
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on C0 IRF_RX on S11
 4) Switch on C6 UART_TX on S11
 5) Switch on C7 UART_RX on S11
 6) Switch on C2 BUZZER on S11
 7) Switch on B3 LED on S11
 8) Insert MC1602 LCD module into LCD_DISPLAY_CONNECTOR
 9) Select COM or USB on JP4
 10) If USB selected, insert UMS1 module into socket
============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>
#include "lcd.h"


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
/* C2C */
/*
#define set_bit(REG,BIT) (REG|=(1<<BIT))
#define clear_bit(REG,BIT) (REG&=~(1<<BIT))
#define bit_is_set(REG,BIT) (REG&(1<<BIT))
*/

/* HITECH */
#define set_bit(REG,BIT) (BIT=1)
#define clear_bit(REG,BIT) (BIT=0)
#define bit_is_set(REG,BIT) (BIT)


#define IR_PULSE_MATCH 16
#define IR_NOISE_CANCEL 200
#define IR_EOP_DETECT 1000
#define IR_REPEAT_TIMEOUT 20000

#define SFH_IN (PORTC&1)

#define EOF -1

#pragma inline timer_init
#pragma inline timer_clear
#pragma inline timer_get

#define timer_ov() (bit_is_set(PIR1,TMR1IF))
#define timer_start() (set_bit(T1CON,TMR1ON))
#define timer_stop() (clear_bit(T1CON,TMR1ON))

#define STATE_0 0
#define STATE_I 1

#define MIN_BITS 16


unsigned int pulse;

const unsigned char hexdigit[16]={'0','1','2','3','4','5','6','7',
                                  '8','9','A','B','C','D','E','F'};


/*============================================================================

       Program

============================================================================*/

/* ----- UART routines ----- */

void uart_init(void)
{
  clear_bit(TXSTA,SYNC);
  set_bit(TXSTA,BRGH);
  SPBRG=20;
  set_bit(RCSTA,SPEN);
  set_bit(RCSTA,CREN);
  set_bit(TXSTA,TXEN);
}

void uart_tx(unsigned char c)
{
  while (!TXIF) ;
  TXREG=c;
}

int uart_rx(void)
{
  if (!RCIF) return EOF;
  return RCREG;
}

unsigned char uart_rx_wait(void)
{
  while (!RCIF);
  return RCREG;
}

/* ----- timer1 routines ----- */

void timer_init(void)
{
  clear_bit(T1CON,TMR1CS);
  clear_bit(T1CON,T1OSCEN);
  set_bit(T1CON,T1CKPS0);
  set_bit(T1CON,T1CKPS1);
}

void timer_clear(void)
{
  TMR1L=0;
  TMR1H=0;
  clear_bit(PIR1,TMR1IF);
}

unsigned int timer_get(void)
{
  unsigned char h=TMR1H;
  unsigned char l=TMR1L;
  if (h==TMR1H) return (unsigned int)h<<8|l;
  return (unsigned int)TMR1H<<8|TMR1L;
}

/* ----- capture code ----- */

unsigned int measure_pulse(unsigned char hl, unsigned int eop)
{
  unsigned int pulse;
  pulse=0;
  while (SFH_IN==hl) {
    pulse=timer_get();
    if (pulse>eop) break;
  } 
  timer_clear();
  return pulse;
}

void measure_mark(void)
{
  do {
    pulse=timer_get();
    if (pulse>IR_EOP_DETECT) break;
  } while (!SFH_IN);
  timer_clear();
}

void measure_space(void)
{
  do {
    pulse=timer_get();
    if (pulse>IR_EOP_DETECT) break;
  } while (SFH_IN);
  timer_clear();
}

unsigned char compare_pulse(unsigned int ref)
{
  if (pulse>ref) return pulse-ref<IR_PULSE_MATCH;
  else return ref-pulse<IR_PULSE_MATCH;
}

/*
 * simple uir capture - 6 bytes code hash
 * does not cancel code not conforming
 * to common scheme
*/

unsigned char capture_uir(unsigned char *code)
{
 unsigned char i;
 unsigned char b;
 unsigned char m;

 unsigned int last_mark=0;
 unsigned int last_space=0;

 for (i=0; i<6; i++) code[i]=0;

 timer_clear();

 measure_mark();
 if (pulse < IR_NOISE_CANCEL) return 0;

 measure_space();
 if (pulse > IR_EOP_DETECT) return 0;

 i=0;
 b=0;
 m=0x80;
 while(1) {
   
   measure_mark();
   if (pulse>IR_EOP_DETECT) break;
   if (!compare_pulse(last_mark)) code[b]^=m;
   last_mark=pulse;
   if (!(m>>=1)) { 
     m=0x80;
     if (++b>5) b=0;
   }
   i++;

   measure_space();
   if (pulse>IR_EOP_DETECT) break;
   if (!compare_pulse(last_space)) code[b]^=m;
   last_space=pulse;
   if (!(m>>=1)) { 
     m=0x80;
     if (++b>5) b=0;
   }
   i++; 

 }

 return i;
}

/* ----- beep ----- */

void beep(unsigned int len, unsigned int tone)
{
  unsigned int i;
  while (len>2*tone) {
    PORTC|=0x04;
    for (i=0; i<tone; i++);
    PORTC&=~0x04;
    for (i=0; i<tone; i++);
    len-=2*tone;
  }
}

/* ----- new data report ----- */

void report_uir(unsigned char *code)
{
  unsigned char i;

  DisplayClear();
  for (i=0; i<6; i++) {
    DisplayWrData(hexdigit[code[i]>>4]);
    DisplayWrData(hexdigit[code[i]&0x0f]);
    uart_tx(code[i]);
  }
  beep(500,200);
}

/* ----- program entry point ----- */

void main(void)
{
  unsigned char state=0;
  unsigned char c;
  unsigned char i;

  unsigned char uir_code[6];
  unsigned char prev_uir_code[6];

  unsigned char bits;
  unsigned char repeat;

  uart_init();

  timer_init();
  timer_start();

  DispHwInit();

  TRISC&=~0x04;

  TRISB=0xE7;
  PORTB&=~0x08;

  TRISD=0;

  while (1) {

    switch (c=uart_rx()) {
      case EOF: break;
      case 'I':
        state=STATE_I;
        break;
      case 'R':
        if (state==STATE_I) {
          uart_tx('O');
          uart_tx('K');
        }
      default:
        state=STATE_0;
    }          
    
    if (!SFH_IN) {
      if (timer_get()>IR_REPEAT_TIMEOUT || timer_ov()) repeat=0;
      PORTB|=0x08;
      bits=capture_uir(uir_code);
      PORTB&=~0x08;
      if (bits>=MIN_BITS) {
        report_uir(uir_code);
        for (i=0; i<6; i++) prev_uir_code[i]=uir_code[i];
        timer_clear();
        repeat=1;
      }
      else if (repeat) {
        report_uir(prev_uir_code);
        timer_clear();
        repeat=1;
      }
    }
  }
}
