/*============================================================================

                        PVK40 Example : Sound recording

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : HS 20.000 MHz

 Serial port : 115200 Bd, 8 db, parity none, 1 stop bit, flow control none

 On your PVK40 board :
 1) Insert 24FC515 into 24xx socket ***
 2) Mount 20 Mhz crystal on XT position
 3) Switch off all DIP switches except:
 4) Switch on MIC on S9
 5) Switch on B3 LED on S11
 6) Switch on C6 UART_TX on S11
 7) If USB selected, insert UMS1 module into socket

 *** Note: pin 3 (A2) must be tied to Vcc
============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & HS); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

#define WavHeaderLen 0x2C
const unsigned char wavheader[] = {0x52,0x49,0x46,0x46,0x24,0x00,0x01,0x00,
                                   0x57,0x41,0x56,0x45,0x66,0x6D,0x74,0x20,
                                   0x10,0x00,0x00,0x00,0x01,0x00,0x01,0x00,
                                   0x40,0x1F,0x00,0x00,0x40,0x1F,0x00,0x00,
                                   0x01,0x00,0x08,0x00,0x64,0x61,0x74,0x61,
                                   0x00,0x00,0x01,0x00 };

static bit led @ PORTBIT(PORTB,3);

volatile bit button0 @ PORTBIT(PORTD,0);
volatile bit button7 @ PORTBIT(PORTD,7);

unsigned char buffer0[64]; /* 64-byte buffer in bank 0 */
bank1 unsigned char buffer1[64]; /* 64-byte buffer in bank 1 */

char buffptr;

char buffer0_rd=0;
char buffer1_rd=0;
char buffer0_wr=0;
char buffer1_wr=0;

/*============================================================================

       Program

============================================================================*/

/* ----- initialisation ----- */

void timer0init(void)
{
 /* initialisation of timer0; */
	PS2=0; PS1=0; PS0=1; /* select prescaler by 4 */
	T0CS=0; /* select timer0 */
	PSA=0;  /* select prescaler for timer0 */
	TMR0=0; /* preload timer0 */
	T0IE=1; /* interrupts are actived */
}

void i2cinit(void)
{
  STAT_CKE=0;
  STAT_SMP=1;
  SSPADD=0x04;  /* setup 1MHz I2C clock */
  SSPCON2=0x00;
  WCOL=0;SSPOV=0;
  CKP=0;
  SSPM3=1;SSPM2=0;SSPM1=0;SSPM0=0; /* I2C master mode */
  SSPEN=1;    /* enable SSP */
  SSPIF=0;
}

void uartinit(void)
{  /* serial port initialisation */
  RCSTA=0x80; /* pin RC6 is automaticaly configured as output */
  SPBRG=0x0A; /* 115200Bd */
  TXSTA=0x26; /* TX enabled */
}

void adcinit(void)
{
  /* initialisation of AD converter */
  TRISA = 0xFF;	/* all pin of portA are inputs */
  ADCON1 = 0x02;  /*config in AN4/0 */
  ADCON0 = 0xA1; /* config the converter  */
}

/* ----- I2C routines ----- */

void i2c_start_bit(void)
{
  SSPIF=0;
  for(SEN=1;!SSPIF;) ;
}

void i2c_stop_bit(void)
{
  SSPIF=0;
  for(PEN=1;!SSPIF;) ;
}

void i2c_restart_bit(void)
{
  SSPIF=0;
  for(RSEN=1;!SSPIF;) ;
}

void i2c_write(unsigned char data)
{  /* send byte to i2c device */
  SSPIF=0;
  SSPBUF=data;
  while (!SSPIF) ;
}

unsigned char i2c_read(void)
{  /* read byte from i2c bus */
  unsigned char temp;

  SSPIF=0;
  RCEN=1;  /* initiate reception of byte */
  while (!SSPIF) ;
  temp=SSPBUF;
  SSPIF=0;
  ACKEN=1; /* generate ACK/NO_ACK bit */
  while (!SSPIF) ;
  return temp;
}

unsigned char eep_read(unsigned int address)
{
  unsigned char tempdata;

  for(;;) {     /* acknowledge pooling loop */
    i2c_start_bit();  /* start bit */
    if(address&0x8000)
      i2c_write(0xA8);  /* control byte - write */
    else
      i2c_write(0xA0);  /* control byte - write */
    if(!ACKSTAT)
      break;
    i2c_stop_bit();   /* stop bit */
  }
  i2c_write((address>>8)&0x7F); /* write address MSB to EEPROM */
  i2c_write(address&0xFF); /* write address LSB to EEPROM */
  i2c_restart_bit();
  if(address&0x8000)
    i2c_write(0xA9);  /* control byte - read */
  else
    i2c_write(0xA1);  /* control byte - read */
  ACKDT=1; /* select NO_ACK bit */
  tempdata=i2c_read();
  i2c_stop_bit(); /* stop bit */
  return tempdata;
}

void eep_write(char bufferindex, unsigned int address)
{
  char i;

  for(;;) {     /* acknowledge pooling loop */
    i2c_start_bit();  /* start bit */
    if(address&0x8000)
      i2c_write(0xA8);  /* control byte - write */
    else
      i2c_write(0xA0);  /* control byte - write */
    if(!ACKSTAT)
      break;
    i2c_stop_bit();   /* stop bit */
  }
  i2c_write((address>>8)&0x7F); /* write address MSB to EEPROM */
  i2c_write(address&0xFF); /* write address LSB to EEPROM */
  for(i=0;i<64;i++) { /* write data */
    if(bufferindex)
      i2c_write(buffer1[i]);  /* write data from buffer 1 to EEPROM */
    else
      i2c_write(buffer0[i]);  /* write data from buffer 0 to EEPROM */
  }
  i2c_stop_bit(); /* stop bit */
}

/* ----- UART routine ----- */

void uart_sendbyte(unsigned char databyte)
{
  while(!TRMT) ; /* wait for operation complete */
  TXREG=databyte;
}

/* ----- delay ----- */

void delay(unsigned int us)
{
  for (;us--;);
}

/* ----- Main code ----- */

void RecordSound(void)
{
  unsigned int eepdatptr=0; /* EEPROM data pointer */

  led=1;
  buffer0_rd=0; /* clear flags */
  buffer0_wr=0;
  buffer1_rd=0;
  buffer1_wr=0;
  ADCON0|=0x04; /* start first conversion */
  TMR0=100;  /* decrease timer0 period to 125usec */
  T0IF=0;
  GIE=1;

  while (1) {
    if(buffer0_rd) {
      eep_write(0,eepdatptr);
      buffer0_rd=0;
      eepdatptr+=64;
      if(!eepdatptr) break;
    }
    if(buffer1_rd) {
      eep_write(1,eepdatptr);
      buffer1_rd=0;
      eepdatptr+=64;
      if(!eepdatptr) break;
    }
  }
  GIE=0;
  led=0;
}

void TransmitSound(void)
{
  unsigned int cnt;

  led=1;
  for(cnt=0;cnt<WavHeaderLen;cnt++) /* header */
    uart_sendbyte(wavheader[cnt]);
  cnt=0;
  do {
    uart_sendbyte(eep_read(cnt));   /* data */
  } while (++cnt);
  led=0;
}

void interrupt timer0(void)
{
  TMR0=100; /* decrease timer0 period to 125usec */
  T0IF=0;
  while(ADCON0&0x04) ; /* wait for ADC done */

  if(buffer0_wr) {              /* writing buffer 0 */
    buffer0[buffptr]=ADRESH;
    if(++buffptr==64) {
      buffer0_wr=0;
      buffer0_rd=1;
    }
  }
  else if(buffer1_wr) {          /* writing buffer 0 */
    buffer1[buffptr]=ADRESH;
    if(++buffptr==64) {
      buffer1_wr=0;
      buffer1_rd=1;
    }
  }
  else if(!buffer0_rd) {      /* start write to buffer 0 */
    buffptr=0;
    buffer0[buffptr++]=ADRESH;
    buffer0_wr=1;
  }
  else if(!buffer1_rd) {      /* start write to buffer 1 */
    buffptr=0;
    buffer1[buffptr++]=ADRESH;
    buffer1_wr=1;
  }
  ADCON0|=0x04; /* start new conversion */
}
 
int main(void)
{
  TRISB=0xF7;
  led=0;
  timer0init();
  i2cinit();
  uartinit();
  adcinit();

  for(;;)
  {
    if(!button0) {
      delay(100);
      while(!button0) ;
      delay(60000);      
      RecordSound();
    }
    if(!button7)
      TransmitSound();
  }
}
